#
#   PROBE - Collected ata
#

from random import normalvariate

#  Category identifiers

class DatumCategory(object):
	#  name    string
	#  abbrev  string
	pass

class CatTemperature(DatumCategory):
	name = "Temperature"
	abbrev = "Temp."

class CatAtmosphericPressure(DatumCategory):
	name = "Atmospheric pressure"
	abbrev = "Atm. press."

class CatAtmosphericComposition(DatumCategory):
	name = "Atmospheric composition"
	abbrev = "Atm. comp."

class CatSurfaceComposition(DatumCategory):
	name = "Surface composition"
	abbrev = "Surf. comp."

#---------------------------------------------------------------------------

class Datum(object):
	#  category
	#  rewarded       boolean
	
	rewarded = False

#---------------------------------------------------------------------------

class DatumDict(dict):

	def add(self, key, datum):
		assert self.get(key) is None, "Adding duplicate item to DatumDict"
		self[key] = datum

	def store(self, key, new_datum):
		old_datum = self.get(key)
		if old_datum:
			#print "DatumDict.store: Updating", old_datum, "with", new_datum ###
			old_datum.merge(new_datum)
		else:
			print "DatumDict.store: Adding", new_datum ###
			self[key] = new_datum

#---------------------------------------------------------------------------

class NumericDatum(Datum):
	#  value      float
	#  accuracy   float   (relative)
	#  format     format string

	def __init__(self, value, accuracy):
		error = normalvariate(0.0, accuracy)
		self.value = value * (1.0 + error)
		self.accuracy = accuracy
	
	def merge(self, other):
		assert isinstance(other, NumericDatum), "Merging incompatible datums"
		if other.accuracy < self.accuracy:
			print "NumericDatum.merge: Updating", self, "with", self ###
			self.value = other.value
			self.accuracy = other.accuracy
			self.rewarded = False
	
	def format_value(self):
		value_str = self.format % self.value
		accu_str = "%.0f" % (100 * self.accuracy)
		return "%s +/- %s%%" % (value_str, accu_str)

	def __repr__(self):
		return "<NumericDatum %s %.3e +/- %s>" % (
			self.category, self.value, 100*self.accuracy)


class TemperatureDatum(NumericDatum):
	category = CatTemperature
	format = "%.1f C"

class AtmosphericPressureDatum(NumericDatum):
	category = CatAtmosphericPressure
	format = "%.1f kPa"

class SubstanceDatum(NumericDatum):
	
	def __init__(self, substance, value, accuracy):
		self.category = substance
		self.value = value
		self.accuracy = accuracy
	
	def __cmp__(self, other):
		return cmp(self.value, other.value)
	
	def format_value(self):
		return "%.1f%% +/- %s%%" % (100 * self.value, 100 * self.accuracy)

#---------------------------------------------------------------------------

class CompositionDatum(Datum):
	#  substances   DatumDict{substance: NumericDatum}
	
	def __init__(self):
		self.substances = DatumDict()
	
	def add(self, substance, datum):
		self.substances.add(substance, datum)
	
	def store(self, substance, datum):
		self.substances.store(substance, datum)
	
	def merge(self, other):
		assert isinstance(other, CompositionDatum), "Merging datums of different types"
		assert self.category == other.category, "Merging datums from different categories"
		#print "CompositionDatum.merge: Updating", self, "with", other ###
		for substance, other_datum in other.substances.iteritems():
			datum = self.substances.get(substance)
			if datum:
				datum.merge(other_datum)

class AtmosphericCompositionDatum(CompositionDatum):
	category = CatAtmosphericComposition

class SurfaceCompositionDatum(CompositionDatum):
	category = CatSurfaceComposition
