#
#   PROBE - Game
#

import random
from albow.resource import get_sound
from system import System
from settings import frame_time, options
from vector_algebra import add, rectangular
from rocket_science import fuel_for_liftoff
from data import DatumDict
import instruments

#data_by_category_dict = autodict(list)
#data_by_planet_dict = audodict(data_by_category_dict)
#data_by_system_dict = autodict(data_by_planet_dict)

class Game(object):
	#  current_system      System
	#  mission_clock       float, s
	#  current_time_scale  float
	#  collected_data      {(system_name, planet_name, category): Datum}
	
	launch_sound = get_sound("countdown-and-launch.ogg")
	time_scales = [1, 10, 60, 600, 3600, 36000, 86400, 864000]
	
	mission_clock = -5.0
	mission_clock_running = False
	current_time_scale = 1.0
	
	def __init__(self):
		random.seed(options.random_seed)
		self.collected_data = DatumDict()

	def new_system(self):
		self.current_system = System()
		if options.test_vehicle:
			print "Game.new_system: creating test vehicle" ###
			home = self.current_system.home
			self.current_system.set_vehicle(create_test_vehicle(home))
		if options.test_data:
			print "Game.new_system: creating test data" ###
			import data
			self.store_datum(home, data.TemperatureDatum(42, 0.5))
			self.store_datum(home, data.AtmosphericPressureDatum(42, 0.4))

	def launch(self):
		if not self.mission_clock_running:
			self.mission_clock_running = True
			if options.test_skip_countdown:
				self.mission_clock = 0.0
			else:
				self.launch_sound.play()
	
	def begin_frame(self):
		dt = frame_time * 1e-3 * self.current_time_scale
		old_clock = self.mission_clock
		new_clock = old_clock
		if self.mission_clock_running:
			new_clock += dt
			self.mission_clock = new_clock
		system = self.current_system
		if system:
			if old_clock <= 0.0 and new_clock > 0.0:
				system.end_countdown()
			system.timestep(self, dt)
	
	def num_timescales(self):
		return len(self.time_scales)
	
	def current_timescale_number(self):
		return self.time_scales.index(self.current_time_scale)
	
	def set_timescale_number(self, n):
		n = max(0, min(self.num_timescales() - 1, n))
		self.current_time_scale = self.time_scales[n]
	
	def decrease_timescale(self):
		self.set_timescale_number(self.current_timescale_number() - 1)

	def increase_timescale(self):
		self.set_timescale_number(self.current_timescale_number() + 1)
	
	def get_datum(self, planet, category):
		#  Get datum about planet of current system
		key = (self.current_system.name, planet.name, category)
		return self.collected_data.get(key)
	
	def store_datum(self, planet, datum):
		#  Stores datum under the name of the current system.
		key = (self.current_system.name, planet.name, datum.category)
		self.collected_data.store(key, datum)
		
#		#print "Game.store_datum:", planet, datum ###
#		old_datum = self.collected_data.get(key)
#		if old_datum:
#			old_datum.merge(datum)
#		else:
#			print "Game.store_datum: storing new datum:", planet, datum ###
#			self.collected_data[key] = datum


def create_test_vehicle(home):
	from vehicle import Vehicle, VehicleStage
	vehicle = Vehicle("Pork Pie 1")
	vehicle.add_instrument(instruments.TemperatureSensor(1e9, 0.20))
	vehicle.add_instrument(instruments.AtmosphericPressureSensor(1e9, 0.10))
	vehicle.add_instrument(instruments.Spectrometer(1e9, 0.10, 0.05))
	#                               m0  Isp      b    f    tb
	vehicle.add_stage(VehicleStage(1000, 300,    1, 1e3))
	vehicle.add_stage(VehicleStage(1000, 350, 10e3,   0, 1e30))
	vehicle.name_stages()
	if options.test_prepare:
		booster = vehicle.stages[-1]
		b = booster.burn_rate
		m = vehicle.mass
		g = home.surface_gravity()
		ve = home.escape_velocity()
		fuel = fuel_for_liftoff(booster.Isp, b, m, g, ve, 0.01)
		vehicle.prepare_for_launch(home, heading = 0, fuel = fuel)
	return vehicle
