#
#   PROBE - Instruments carried by a probe
#

import data

class Instrument(object):
	#  name   string
	pass

#---------------------------------------------------------------------------

class NumericInstrument(Instrument):
	#  range          float   m
	#  accuracy       float   (relative)
	#  datum_class    type
	
	def __init__(self, range, accuracy):
		self.range = range
		self.accuracy = accuracy

	def measure(self, result, planet, distance):
		#print "NumericInstrument.measure:", planet, "%.3e" % distance ###
		if distance > self.range:
			return
		value = self.measure_value(planet)
		if value is not None:
			#print "NumericInstrument.measure: value =", value ###
			result.append(self.datum_class(value, self.accuracy))


class TemperatureSensor(NumericInstrument):
	name = "Temperature sensor"
	#category = "Temperature"
	datum_class = data.TemperatureDatum
	
	def measure_value(self, planet):
		#print "TemperatureSensor.measure_value:", planet ###
		return getattr(planet, 'temperature', None)


class AtmosphericPressureSensor(NumericInstrument):
	name = "Atmospheric pressure sensor"
	#category = "Atmospheric pressure"
	datum_class = data.AtmosphericPressureDatum

	def measure_value(self, planet):
		return getattr(planet, 'atmospheric_pressure', None)

#---------------------------------------------------------------------------

class Spectrometer(Instrument):
	#  range          float   m
	#  accuracy       float   (relative)
	#  sensitivity    float   (proportion)

	name = "Spectrometer"
	
	def __init__(self, range, accuracy, sensitivity):
		self.range = range
		self.accuracy = accuracy
		self.sensitivity = sensitivity
	
	def measure(self, result, planet, distance):
		#print "Spectrometer.measure:", planet, "%.3e" % distance ###
		if distance <= self.range:
			self.measure_substances(result,
				data.AtmosphericCompositionDatum,
				getattr(planet, 'atmospheric_composition', None))
			self.measure_substances(result,
				data.SurfaceCompositionDatum,
				getattr(planet, 'surface_composition', None))

	def measure_substances(self, result, datum_class, composition):
		#print "Spectrometer.measure_substances:", composition ###
		if composition:
			comp_datum = datum_class()
			for item in composition:
				#print "...", item.substance.name, item.fraction, self.sensitivity ###
				if item.fraction >= self.sensitivity:
					frac_datum = data.SubstanceDatum(item.substance, item.fraction, self.accuracy)
					#print "......frac_datum =", frac_datum ###
					comp_datum.add(item.substance, frac_datum)
			result.append(comp_datum)
