#
#   PROBE - Scientific data display
#

from albow.widget import Widget
from albow.resource import get_image, get_font
from text_writer import TextWriter
import data

class SciDataDisplay(Widget):
	#  game   Game
	
	bg_image = get_image("lineflow-200x24.png")
	#bg_color = (255, 255, 255)
	font = get_font(10, "VeraMono.ttf")
	text_color = (0, 0, 0)
	left_margin = 11
	top_margin = 11
	label_columns = 12

	def __init__(self, rect, game):
		Widget.__init__(self, rect)
		self.game = game
		print "SciDataDisplay: line height =", self.font.get_linesize() ###
	
	def draw(self, surf):
		frame = surf.get_rect()
		#surf.blit(self.bg_image, (0, 0))
		bg = self.bg_image
		h = bg.get_height()
		y = 0
		while y < frame.height:
			surf.blit(bg, (0, y))
			y += h
		clip = frame.inflate(-self.left_margin, -self.top_margin)
		surf.set_clip(clip)
		game = self.game
		w = TextWriter(surf,
			font = self.font, color = self.text_color,
			x = self.left_margin, y = self.top_margin)
		system = game.current_system
		if system:
			planet = system.selection
			if planet:
				w.writeln(planet.name.upper())
				w.newline()
				self.display_numeric_data(w, planet, data.CatTemperature)
				self.display_numeric_data(w, planet, data.CatAtmosphericPressure)
				self.display_atmospheric_composition(w, planet)
				self.display_surface_composition(w, planet)
	
	def display_atmospheric_composition(self, w, planet):
		if planet.has_atmospheric_composition:
			w.newline()
			if planet.atmospheric_composition:
				w.writeln("Atmosphere")
				self.display_composition(w, planet, data.CatAtmosphericComposition)
			else:
				w.writeln("No atmosphere")
	
	def display_surface_composition(self, w, planet):
		if planet.has_surface_composition:
			w.newline()
			w.writeln("Composition")
			self.display_composition(w, planet, data.CatSurfaceComposition)

	def display_numeric_data(self, w, planet, category):
		datum = self.game.get_datum(planet, category)
		if datum:
			value = datum.format_value()
		else:
			value = "Unknown"
		w.writeln("%-*s %s" % (self.label_columns, category.abbrev, value))

	def display_composition(self, w, planet, category):
		main_datum = self.game.get_datum(planet, category)
		if main_datum:
			data = main_datum.substances.values()
			data.sort()
			data.reverse()
			unknown = 1.0
			lcw = self.label_columns - 2
			for datum in data:
				w.writeln("  %-*s %s" % (lcw, datum.category.name, datum.format_value()))
				unknown -= datum.value
			if unknown > 0.0:
				w.writeln("  %-*s %.1f%%" % (lcw, "Unknown", 100 * unknown))
		else:
			w.writeln("  Unknown")
