#
#   PROBE - Game shell
#

import sys
from pygame.locals import K_ESCAPE
from albow.shell import Shell
from game import Game
import settings
from settings import options
from intro_screen import IntroScreen
from new_system_screen import NewSystemScreen
from mission_planning_screen import MissionPlanningScreen
from launch_params_screen import LaunchParamsScreen
from mission_control_screen import MissionControlScreen

class ProbeShell(Shell):

	def __init__(self, display):
		Shell.__init__(self, display)
		self.game = Game()
		self.game.new_system()
		self.intro_screen = IntroScreen(self)
		self.new_system_screen = NewSystemScreen(self)
		self.mission_planning_screen = MissionPlanningScreen(self)
		self.launch_params_screen = LaunchParamsScreen(self)
		self.mission_control_screen = MissionControlScreen(self)
		self.set_timer(settings.frame_time)
		if options.test_mission_planning:
			self.go_to_mission_planning()
		elif options.test_launch_params:
			self.go_to_launch_prep()
		elif options.test_mission_control:
			self.go_to_mission_control()
		else:
			self.set_timer(settings.intro_frame_time)
			self.show_screen(self.intro_screen)
	
	def show_screen(self, screen):
		Shell.show_screen(self, screen)
		screen.focus()

	def begin_frame(self):
		self.game.begin_frame()
		self.current_screen.begin_frame()

	def end_intro_screen(self):
		self.set_timer(settings.frame_time)
		self.show_screen(self.new_system_screen)

	def go_to_mission_planning(self):
		self.show_screen(self.mission_planning_screen)
	
	def go_to_launch_prep(self):
		self.show_screen(self.launch_params_screen)

	def go_to_mission_control(self):
		self.show_screen(self.mission_control_screen)

	def key_down(self, e):
		if e.key == K_ESCAPE:
			sys.exit(0)
