#
#   PROBE - Substances to be found in a planet or its atmosphere
#

from random import uniform, shuffle

class Substance(object):
	#  name    string  [key]
	#  abbrev  string
	
	def __init__(self, name, abbrev):
		self.name = name
		self.abbrev = abbrev
	
	def __cmp__(self, other):
		return cmp(self.name, other.name)

	def __hash__(self):
		return hash(self.name)

class SubstanceFraction(object):
	#  fraction    float     0.0 to 1.0
	#  substance   Substance
	
	def __init__(self, fraction, substance):
		self.fraction = fraction
		self.substance = substance
	
	def __iter__(self):
		yield self.fraction
		yield self.substance

	def __cmp__(self, other):
		return cmp(self.fraction, other.fraction)

	def __hash__(self):
		return hash(self.fraction)

hydrogen = Substance("Hydrogen", "H2")
helium = Substance("Helium", "He")
argon = Substance("Argon", "Ar")
carbon = Substance("Carbon", "C")
nitrogen = Substance("Nitrogen", "N2")
oxygen = Substance("Oxygen", "O2")
sulphur = Substance("Sulphur", "S2")
nickel = Substance("Nickel", "Ni")
iron = Substance("Iron", "Fe")
lead = Substance("Lead", "Pb")
copper = Substance("Copper", "Cu")
uranium = Substance("Uranium", "U")

water_vapour = Substance("Water vapour", "H2O/G")
water = Substance("Water", "H2O/L")
ice = Substance("Ice", "H2O/S")
carbon_dioxide = Substance("Carbon dioxide", "CO2")
methane = Substance("Methane", "CH4")
ammonia = Substance("Ammonia", "NH3")
nitrous_oxide = Substance("Nitrous oxide", "NO2")
ozone = Substance("Ozone", "O3")
sulphuric_acid = Substance("Sulphuric acid", "H2SO4")
nitric_acid = Substance("Nitric acid", "HNO3")
hydrogen_chloride = Substance("Hydrogen chloride", "HCl")
sodium_chloride = Substance("Sodium chloride", "NaCl")
silicon_dioxide = Substance("Silicon dioxide", "SiO2")
calcium_carbonate = Substance("Calcium carbonate", "CaCO3")

gas_giant_atmosphere_substances = [
	hydrogen, helium, methane, ammonia, carbon_dioxide, water_vapour,
	nitrogen,
]

rocky_planet_atmosphere_substances = [
	nitrogen, water_vapour, carbon_dioxide, nitrous_oxide,
	sulphuric_acid, nitric_acid, hydrogen_chloride,
	helium, argon,
]

rocky_planet_surface_substances = [
	carbon, sulphur, nickel, iron, lead, copper, uranium,
	sulphuric_acid, nitric_acid, sodium_chloride, silicon_dioxide,
	calcium_carbonate, ice
]

terrestrial_planet_atmosphere_substances = [
	oxygen, nitrogen, water_vapour, carbon_dioxide, ozone,
	helium, argon,
]

terrestrial_planet_surface_substances = [
	carbon, sulphur, nickel, iron, lead, copper, uranium,
	sodium_chloride, silicon_dioxide, calcium_carbonate, ice
]

def generate_composition(choices):
	result = []
	deck = choices[:]
	shuffle(deck)
	unallocated = 1.0
	while deck and unallocated > 0.001:
		amount = uniform(0.001, unallocated)
		result.append(SubstanceFraction(amount, deck.pop()))
		unallocated -= amount
	result.sort()
	return result

def dump_composition(title, items):
	if items:
		print title
		for amount, subst in items:
			print "   %-20s %6.2f%%" % (subst.name, 100 * amount)
