#
#   PROBE - Text formatting utilities
#

from textwrap import wrap

class TextWriter(object):

	def __init__(self, surface, font, color, x, y):
		self.surface = surface
		self.font = font
		self.color = color
		self.left_margin = x
		self.x = x
		self.y = y
	
	def write(self, text):
		buf = self.font.render(text, True, self.color)
		self.surface.blit(buf, (self.x, self.y))
		self.x += buf.get_width()
	
	def writeln(self, text):
		self.write(text)
		self.newline()
	
	def newline(self):
		self.x = self.left_margin
		self.y += self.font.get_linesize()
	
	def write_lines(self, text):
		for line in text.split("\n"):
			self.writeln(line)

	def write_paragraph(self, text, width):
		for line in wrap(text, width):
			self.writeln(line)
	
	def write_paragraphs(self, text, width):
		for para in text.split("\n\n"):
			self.write_paragraph(para, width)
