#
#   PROBE - Tracking and control display
#

from albow.widget import Widget
from albow.resource import get_font
from text_writer import TextWriter
from vector_algebra import subtract, magnitude, direction

class TrackingDisplay(Widget):

	font = get_font(10, "VeraMono.ttf")
	text_color = (0, 255, 0)
	left_margin = 20
	top_margin = 20

	def __init__(self, rect, game):
		Widget.__init__(self, rect)
		self.game = game
	
	def draw(self, surf):
		system = self.game.current_system
		if system:
			vehicle = system.vehicle
			if vehicle:
				frame = surf.get_rect()
				w = TextWriter(surf, font = self.font, color = self.text_color,
					x = self.left_margin, y = self.top_margin)
				refbody = system.selection or system.home
				pos = subtract(vehicle.position, refbody.position)
				altitude = magnitude(pos) - refbody.radius
				azimuth = direction(pos)
				w.writeln("Position from %s" % refbody.name)
				w.writeln("  Altitude   %.3e km" % (1e-3 * altitude))
				w.writeln("  Azimuth    %07.3f deg" % azimuth)
				vel = vehicle.velocity
				w.writeln("Velocity     %.3e km/s" % (1e-3 * magnitude(vel)))
				w.writeln("Heading      %05.1f deg" % direction(vel))
				w.writeln("Attitude     %05.1f deg" % vehicle.attitude)
				lbl = "Fuel:"
				for stage in vehicle.stages:
					w.writeln("%-5s %3s %8.0f kg" % (lbl, stage.name, stage.fuel))
					lbl = ""
				if vehicle.state == vehicle.FLYING:
					stage = vehicle.current_stage()
					burn_time = stage.burn_time
					if burn_time:
						w.writeln("BURN   %3s  %5.1f s" % (stage.name, stage.burn_time))
